"""计算rsr得分"""
function rsr(df::DataFrame)::Matrix
    m, n = size(df)
    result = Matrix{Float64}(undef, m, n)
    for j in 1:n
        compare_list = sort(df[:, j])
        for i in 1:m
            result[i, j] = searchsortedfirst(compare_list, df[i, j])
        end
    end
    result
end


"""计算ni_rsr得分"""
function ni_rsr(df::DataFrame)::Matrix
    m, n = size(df)
    result = Matrix{Float64}(undef, m, n)
    for j in 1:n
        max_x = maximum(df[!, j])
        min_x = minimum(df[!, j])
        result[:, j] = 1 .+ (m - 1) .* ((df[!, j] .- min_x) ./ (max_x - min_x))
    end
    return result
end


"""计算topsis得分"""
function topsis(df::DataFrame, weight=critic)::Matrix
    df = scaling(df)
    weights = weight(df)
    m, n = size(df)
    result = Matrix{Float64}(undef, m, 1)
    D1 = Matrix{Float64}(undef, m, n)
    D2 = Matrix{Float64}(undef, m, n)
    for j in 1:n
        col_max = maximum(df[!, j])
        col_min = minimum(df[!, j])
        for i in 1:m
            D1[i, j] = weights[j] * (df[i, j] - col_max)^2
            D2[i, j] = weights[j] * (df[i, j] - col_min)^2
        end
    end

    for i in 1:m
        result[i] = sqrt(sum(D1[i, :])) / (sqrt(sum(D1[i, :])) + sqrt(sum(D2[i, :])))
    end
    return result
end