"""正向指标转化为负向指标"""
function tiny_convert(df::DataFrame, mode::String, change_list::Vector{Int64})::DataFrame
    result = copy(df)
    if mode == "0"
        for j in change_list
            result[!, j] = 1 ./ df[!, j]
        end
    elseif mode == "1"
        for j in change_list
            result[!, j] = df[!, j].max() .- df[!, j]
        end
    else
        println("请正确地选择模式！")
    end
    return result
end


"""中间型指标转化为正向指标"""
function middle_convert(df::DataFrame, change_list::Vector{Int64}, best_value::Vector{Int64})::DataFrame
    result = copy(df)
    for j in change_list
        M = maximum(abs.(result[!, j] .- getindex(best_value, fdindex(change_list, j))))
        result[!, j] = 1 .- abs.(result[!, j] .- getindex(best_value, fdindex(change_list, j))) ./ M
    end
    return result
end

function middle_convert(df::DataFrame, change_list::Vector{Int64}, best_value::Vector{Float64})::DataFrame
    result = copy(df)
    for j in change_list
        M = maximum(abs.(result[!, j] .- getindex(best_value, fdindex(change_list, j))))
        result[!, j] = 1 .- abs.(result[!, j] .- getindex(best_value, fdindex(change_list, j))) ./ M
    end
    return result
end


"""moderate_convert函数的辅助函数"""
function mod_con(x::Real, low::Real, high::Real, M::Real)
    @assert low <= high "下限高于上限"
    if x < low
        return 1 - (low - x) / M
    elseif x > high
        return 1 - (x - high) / M
    else
        return 1
    end
end

"""适度型指标转化为正向指标"""
function moderate_convert(df::DataFrame, change_list::Vector{Int64}, low_limit::Vector{Float64}, high_limit::Vector{Float64})::DataFrame
    result = copy(df)
    for j in change_list
        a = getindex(low_limit, fdindex(change_list, j))
        b = getindex(high_limit, fdindex(change_list, j))
        @assert a <= b "下限高于上限"
        left = maximum(a .- result[!, j])
        right = maximum(result[!, j] .- b)
        M = maximum([left, right])
        result[!, j] = mod_con.(result[!, j], a, b, M)
    end
    return result
end

function moderate_convert(df::DataFrame, change_list::Vector{Int64}, low_limit::Vector{Int64}, high_limit::Vector{Int64})::DataFrame
    result = copy(df)
    for j in change_list
        a = getindex(low_limit, fdindex(change_list, j))
        b = getindex(high_limit, fdindex(change_list, j))
        @assert a <= b "下限高于上限"
        left = maximum(a .- result[!, j])
        right = maximum(result[!, j] .- b)
        M = maximum([left, right])
        result[!, j] = mod_con.(result[!, j], a, b, M)
    end
    return result
end

function moderate_convert(df::DataFrame, change_list::Vector{Int64}, low_limit::Vector{Float64}, high_limit::Vector{Int64})::DataFrame
    result = copy(df)
    for j in change_list
        a = getindex(low_limit, fdindex(change_list, j))
        b = getindex(high_limit, fdindex(change_list, j))
        @assert a <= b "下限高于上限"
        left = maximum(a .- result[!, j])
        right = maximum(result[!, j] .- b)
        M = maximum([left, right])
        result[!, j] = mod_con.(result[!, j], a, b, M)
    end
    return result
end

function moderate_convert(df::DataFrame, change_list::Vector{Int64}, low_limit::Vector{Int64}, high_limit::Vector{Float64})::DataFrame
    result = copy(df)
    for j in change_list
        a = getindex(low_limit, fdindex(change_list, j))
        b = getindex(high_limit, fdindex(change_list, j))
        @assert a <= b "下限高于上限"
        left = maximum(a .- result[!, j])
        right = maximum(result[!, j] .- b)
        M = maximum([left, right])
        result[!, j] = mod_con.(result[!, j], a, b, M)
    end
    return result
end


"""指标归一化"""
function normal(df::DataFrame)::DataFrame
    m, n = size(df)
    result = Matrix{Float64}(undef, m, n)
    for j in 1:n
        col_max = maximum(df[!, j])
        col_min = minimum(df[!, j])
        result[:, j] = (df[!, j] .- col_min) ./ (col_max - col_min)
    end
    return DataFrame(result, :auto)
end


"""指标平均归一化"""
function ave_normal(df::DataFrame)::DataFrame
    m, n = size(df)
    result = Matrix{Float64}(undef, m, n)
    for j in 1:n
        col_max = maximum(df[!, j])
        col_min = minimum(df[!, j])
        col_mean = mean(df[!, j])
        result[:, j] = (df[!, j] .- col_mean) ./ (col_max - col_min)
    end
    return DataFrame(result, :auto)
end


"""指标标准化"""
function standard(df::DataFrame)::DataFrame
    m, n = size(df)
    result = Matrix{Float64}(undef, m, n)
    for j in 1:n
        col_mean = mean(df[!, j])
        col_std = std(df[!, j])
        result[:, j] = (df[!, j] .- col_mean) ./ col_std
    end
    return DataFrame(result, :auto)
end


"""向量归一化"""
function scaling(df::DataFrame)::DataFrame
    m, n = size(df)
    result = Matrix{Float64}(undef, m, n)
    for j in 1:n
        model_length = sum(df[!, j] .^ 2)
        result[:, j] = df[!, j] ./ sqrt(model_length)
    end
    return DataFrame(result, :auto)
end